﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ThinkGeo.MapSuite.DesktopEdition;
using ThinkGeo.MapSuite.Core;
using System.IO;
using System.Windows.Forms;
using System.Collections.ObjectModel;


namespace SphericalMercatorGridTest
{
    class GridFunctions
    {

        public static Dictionary<PointShape, double> GetBathyPoints(string textFilename, Proj4Projection proj4Projection)
        {
            StreamReader input = new StreamReader(textFilename);
            string[] values = new string[3];
            string line = string.Empty;
            PointShape pointShape = new PointShape();
            PointShape nextPoint = new PointShape();

            // Store the points in a collection
            Dictionary<PointShape, double> bathyPoints = new Dictionary<PointShape, double>();
            MultipointShape bathyPointShape = new MultipointShape();
            List<double> bathyDepths = new List<double>();

            while (input.EndOfStream == false)
            {
                line = input.ReadLine();

                char isnumber = char.Parse(line.Substring(0, 1));
                if (char.IsNumber(isnumber) == true || isnumber.ToString() == "-")
                {

                    // Handles text with space, comma or tab character
                    values = line.Split(new Char[] { ' ', ',', '\t' });

                    // Cast the values to a point shape so we can reproject them
                    pointShape.X = Convert.ToDouble(values[0]);
                    pointShape.Y = Convert.ToDouble(values[1]);

                    // ##################################################################
                    // COMMENT THIS LINE AND THEN COMPARE UTM RESULTS WITH SM Projection version
                    // WHEN DATA ARE IN UTM, THEY LOOK OKAY, BUT IF WE WORK IN SM, the INTERPOLATIONS
                    // ARE DIFFERENT
                    pointShape = ConvertUTMToSMCoordinate(pointShape, proj4Projection);
                    // ##################################################################

                    bathyPointShape.Points.Add(new PointShape(pointShape.X, pointShape.Y));

                    double depth = Math.Abs(double.Parse(values[2]));
                    bathyDepths.Add(depth);

                }
            }

            input.Close();

            // Merge the depths and points into bathyPoints dictionary
            int i = 0;
            foreach (PointShape point in bathyPointShape.Points)
            {
                bathyPoints.Add(point, bathyDepths[i]);
                i++;
            }

            return bathyPoints;

        }

        public static GridFeatureLayer MakeGenericBathyGrid(WinformsMap winformsMap, string gridFilename, 
            Proj4Projection proj4Projection, double areaBuffer, Dictionary<PointShape, double> bathyPoints)
        {

            // Find the real min and max values for the dataset and don't process any more than that.
            double minX = bathyPoints.Keys.Min(p => p.X) - areaBuffer;
            double minY = bathyPoints.Keys.Min(p => p.Y) - areaBuffer;
            double maxX = bathyPoints.Keys.Max(p => p.X) + areaBuffer;
            double maxY = bathyPoints.Keys.Max(p => p.Y) + areaBuffer;

            double minDepth = bathyPoints.Values.Min();
            double maxDepth = bathyPoints.Values.Max();

            RectangleShape currentDrawingExtent = new RectangleShape(minX, maxY, maxX, minY);
            currentDrawingExtent.ScaleUp(20);

            //Greate the grid definition based on the extent, cell size etc.
            GridDefinition gridDefinition = new GridDefinition(currentDrawingExtent, 1000, -9999, bathyPoints);

            winformsMap.CurrentExtent = currentDrawingExtent;

            FileStream gridFileStream = null;
            try
            {
                //double tempOuttow = 50000;
                gridFileStream = new FileStream(gridFilename, FileMode.Create, FileAccess.ReadWrite);
                GridFeatureSource.GenerateGrid(gridDefinition, new InverseDistanceWeightedGridInterpolationModel(2, areaBuffer), gridFileStream);
                //GridFeatureSource.GenerateGrid(gridDefinition, new InverseDistanceWeightedGridInterpolationModel(2, tempOuttow), gridFileStream);
            }
            catch (Exception exp)
            {
                MessageBox.Show(exp.Message);
                return null;
            }
            finally
            {
                if (gridFileStream != null) { gridFileStream.Close(); }
            }

            string layerName = new FileInfo(gridFilename).Name;
            //Load a new GridFeatureLayer based on the current grid file
            GridFeatureLayer gridFeatureLayer = new GridFeatureLayer(gridFilename);

            //gridFeatureLayer.FeatureSource.Open();
            //gridFeatureLayer.FeatureSource.Projection = proj4Projection;

            GeoColor startColor = GeoColor.StandardColors.Red;
            GeoColor endColor = GeoColor.StandardColors.Blue;

            //Create a series of colors from blue to red that we will use for the breaks
            Collection<GeoColor> colors = GeoColor.GetColorsInQualityFamily(startColor, endColor,
                10, ColorWheelDirection.Clockwise);

            if (maxDepth - minDepth == 0)
            {
                minDepth -= 50;
                maxDepth += 50;
            }

            Collection<double> isolineLevels = GenerateIsoLineLevels(Convert.ToDecimal(minDepth),
                Convert.ToDecimal(maxDepth), Convert.ToDecimal((maxDepth - minDepth) / 10));

            //Create a class break style
            ClassBreakStyle classBreakLineStyle = new ClassBreakStyle(gridFeatureLayer.DataValueColumnName);

            int contoursCount = colors.Count;

            for (int j = 0; j < contoursCount; j++)
            {
                AreaStyle style = new AreaStyle(new GeoPen(GeoColor.FromArgb(50, colors[j]), 1),
                    new GeoSolidBrush(GeoColor.FromArgb(50, colors[j])));
                classBreakLineStyle.ClassBreaks.Add(new ClassBreak(isolineLevels[j], style));
            }

            gridFeatureLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(classBreakLineStyle);
            gridFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            return gridFeatureLayer;


        }

        public static Collection<double> GenerateIsoLineLevels(decimal min, decimal max, decimal interval)
        {

            // First, we adjust the min and max values down/up according to the interval

            Decimal minDecimal = Decimal.Remainder(min, interval);
            min = min - minDecimal;

            Decimal maxDecimal = Decimal.Remainder(max, interval);
            //max = max - maxDecimal + interval;
            max = max - maxDecimal;

            Collection<double> levels = new Collection<double>();
            for (decimal i = min; i <= max; i = i + interval)
            {
                levels.Add(Convert.ToDouble(i));
            }

            return levels;
        }

        public static RectangleShape ConvertSMToUTMCoordinate(RectangleShape box, Proj4Projection utmProjection)
        {
            // Convert the feature to Lat / Long from Spherical Mercator
            Proj4Projection mgdproj4Projection = new Proj4Projection();
            mgdproj4Projection.InternalProjectionParametersString = utmProjection.ExternalProjectionParametersString;
            mgdproj4Projection.ExternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();
            mgdproj4Projection.Open();
            RectangleShape geoBox = mgdproj4Projection.ConvertToInternalProjection(box);
            mgdproj4Projection.Close();

            return geoBox;

        }

        public static MultipointShape ConvertSMToUTMCoordinate(MultipointShape multipoint, Proj4Projection utmProjection)
        {
            // Convert the feature to Lat / Long from Spherical Mercator
            Proj4Projection mgdproj4Projection = new Proj4Projection();
            mgdproj4Projection.InternalProjectionParametersString = utmProjection.ExternalProjectionParametersString;
            mgdproj4Projection.ExternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();
            mgdproj4Projection.Open();
            BaseShape geoBaseShape = mgdproj4Projection.ConvertToInternalProjection((BaseShape)multipoint);
            mgdproj4Projection.Close();

            return new MultipointShape(geoBaseShape.GetWellKnownText());

        }

        public static RectangleShape ConvertUTMToSMCoordinate(RectangleShape box, Proj4Projection utmProjection)
        {
            // Convert the feature to Spherical Mercator from UTM
            Proj4Projection mgdproj4Projection = new Proj4Projection();
            mgdproj4Projection.InternalProjectionParametersString = utmProjection.ExternalProjectionParametersString;
            mgdproj4Projection.ExternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();
            mgdproj4Projection.Open();
            RectangleShape geoBox = mgdproj4Projection.ConvertToExternalProjection(box);
            mgdproj4Projection.Close();

            return geoBox;

        }

        public static MultipointShape ConvertUTMToSMCoordinate(MultipointShape points, Proj4Projection utmProjection)
        {
            // Convert the feature to Spherical Mercator from UTM 
            Proj4Projection mgdproj4Projection = new Proj4Projection();
            mgdproj4Projection.InternalProjectionParametersString = utmProjection.ExternalProjectionParametersString;
            mgdproj4Projection.ExternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();
            mgdproj4Projection.Open();
            BaseShape geoBaseShape = mgdproj4Projection.ConvertToExternalProjection((BaseShape)points);
            mgdproj4Projection.Close();

            return new MultipointShape(geoBaseShape.GetWellKnownText());

        }

        public static PointShape ConvertUTMToSMCoordinate(PointShape point, Proj4Projection utmProjection)
        {
            // Convert the feature to Spherical Mercator from UTM 
            Proj4Projection mgdproj4Projection = new Proj4Projection();
            mgdproj4Projection.InternalProjectionParametersString = utmProjection.ExternalProjectionParametersString;
            mgdproj4Projection.ExternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();
            mgdproj4Projection.Open();
            Vertex geoVertex = mgdproj4Projection.ConvertToExternalProjection(point.X, point.Y);
            mgdproj4Projection.Close();

            return new PointShape(geoVertex);

        }


    }
}
